/*
 * USB-HID-Maus/Mauswackler
 *
 * Meldet sich am PC als USB-Maus an und sendet alle paar Sekunden eine kleine
 * Bewegung des Mauszeigers
 *
 * Schaltung: STM32 minimal development board oder Bluepill
 * MCU: STM32F103C8T6
 * Takt: 48MHz
 *
 * Keine Einstellungen, keine Hardwaremodifikationen.
 *
 * 09.01.2015 Nicolas
 */


/* Die Firmware ist eine kleine Abwandlung des "Joystick-Demos" fuer das
 * STM3210E-Demo-Boards mit Anpassungen fuer das Mini-Eval Board und fuer
 * 42 MHz Takt.
 *
 * Wenn USB-Inititialisierung erfolgreich, sollte an PB7 ein Rechtecksignal mit
 * der Periodendauer 2*MOUSE_STEP_WAIT_MS anliegen.
 *
 * Kleine Falle: Nach dem Flashen muss der USB neu vom PC enumeriert werden.
 * Deswegen muss das Board entweder ausgesteckt oder Jumper P2 abgezogen werden.
 *  */



#include "hw_config.h"
#include "usb_lib.h"
#include "usb_pwr.h"
#include "stm32f10x_conf.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_gpio.h"
#include <stdint.h>
#include <limits.h>

/* Einstellungen */
#define SPARE1_GPIO GPIOB
#define SPARE1_Pin GPIO_Pin_7
#define SPARE1_RCC RCC_APB2Periph_GPIOB

#define MOUSE_CURSOR_STEP_X 2
#define MOUSE_CURSOR_STEP_Y 0

#define MOUSE_STEP_WAIT_MS 10000UL



__IO uint8_t PrevXferComplete = 1; // Aus dem Joystick-Demo: USB frei

/** Es ist wieder Zeit, die Maus zu bewegen. Wird gesetzt in SysTick, wird geloescht in der
 * Hauptschleife */
bool gl_movemouse = FALSE;


void initspare1(void);
void togglespare1(void);
void mouse_send(void);




int main(void)
{
	SystemInit();
	SysTick_Config(SystemCoreClock/1000); /* 1ms */


	/* USB vorbereiten und starten */
	Set_System();
	USB_Interrupts_Config();
	Set_USBClock();
	USB_Init();


	/* Debug-Pin */
	initspare1();


	/* Ab und an mal einen USB-Frame senden */
	while(1)
	{
		if (bDeviceState == CONFIGURED && gl_movemouse && PrevXferComplete)
		{
			mouse_send();
			gl_movemouse = FALSE;
			togglespare1();

		}
	}
}



/** SysTick-Handler: 1 Millisekunde
 *
 * Setzt alle 10000 Millisekunden das Flag gl_movemouse. */
void SysTick_Handler(void)
{
	static uint32_t counter=0;
	static uint32_t initSteps = 4;

	if(!counter--)
    {
		gl_movemouse = TRUE;
		if( initSteps )
        {
            counter = 250;
            initSteps--;
        }
        else
        {
            counter = MOUSE_STEP_WAIT_MS;
        }
	}
}



/** Sendet abwechselnd einen Mausschritt in unterschiedliche Richtungen.
 *
 * Direkt angelehnt an "Joystick_Send()" aus dem Demo (siehe hwconfig.c) */
void mouse_send(void)
{
	uint8_t Mouse_Buffer[4] = {0, 0, 0, 0};
	int8_t X = 0, Y = 0;
	static bool direction = 0;

    if( direction )
    {
    	X += MOUSE_CURSOR_STEP_X;
    	Y += MOUSE_CURSOR_STEP_Y;
    	direction = 0;
    }
    else
    {
    	X -= MOUSE_CURSOR_STEP_X;
    	Y -= MOUSE_CURSOR_STEP_Y;
    	direction = 1;
    }

    /* prepare buffer to send */
    Mouse_Buffer[1] = X;
    Mouse_Buffer[2] = Y;

    /* Reset the control token to inform upper layer that a transfer is ongoing */
    PrevXferComplete = 0;

    /* Copy mouse position info in ENDP1 Tx Packet Memory Area*/
    USB_SIL_Write(EP1_IN, Mouse_Buffer, 4);

    /* Enable endpoint for transmission */
    SetEPTxValid(ENDP1);
}



/** Pin SPARE 1: Kleiner Helfer zum Debuggen */
void initspare1(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	RCC_APB2PeriphClockCmd(SPARE1_RCC,ENABLE);

	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_InitStructure.GPIO_Pin   = SPARE1_Pin;
	GPIO_Init(SPARE1_GPIO, &GPIO_InitStructure);
}



/** Kleiner Helfer zum Debuggen: Toggelt Pin SPARE1
 *
 * Es scheint ueber die Standard peripheral library keinen Werg zu geben, einen Pin direkt zu
 * toggeln, deswegen die umstaendliche Variante. */
void togglespare1(void)
{
	if(GPIO_ReadInputDataBit(SPARE1_GPIO, SPARE1_Pin))
    {
		GPIO_ResetBits(SPARE1_GPIO, SPARE1_Pin);
	}
	else
	{
		GPIO_SetBits(SPARE1_GPIO, SPARE1_Pin);
	}
}



void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {}
}
